import { IRepoPlugin, IKeyValue, IModel, Repo, ICoordinator, ICoordinatorOptions, PluginEventType, IFinderPlugin, IModelMapper } from 'typestore';
import { IndexedDBPlugin } from "./IndexedDBPlugin";
import Dexie from "dexie";
/**
 * Super simple plain jain key for now
 * what you send to the constructor comes out the
 * other end
 *
 * just like poop!
 */
export declare class IndexedDBKeyValue implements IKeyValue {
    args: any[];
    indexedDBKey: boolean;
    constructor(...args: any[]);
}
export declare class IndexedDBRepoPlugin<M extends IModel> implements IRepoPlugin<M>, IFinderPlugin {
    private store;
    repo: Repo<M>;
    type: number;
    supportedModels: any[];
    private coordinator;
    private keys;
    /**
     * Construct a new repo/store
     * manager for a given repo/model
     *
     * @param store
     * @param repo
     */
    constructor(store: IndexedDBPlugin, repo: Repo<M>);
    /**
     * Create a finder method with descriptor
     * and signature
     *
     * @param repo
     * @param finderKey
     * @returns {any}
     */
    decorateFinder(repo: Repo<any>, finderKey: string): (...args: any[]) => Promise<M | M[]>;
    /**
     * Handle a plugin event
     *
     * @param eventType
     * @param args
     * @returns {boolean}
     */
    handle(eventType: PluginEventType, ...args: any[]): boolean | any;
    /**
     * Model mapper
     *
     * @returns {IModelMapper<M>}
     */
    readonly mapper: IModelMapper<M>;
    /**
     * Get dexie table
     *
     * @returns {Dexie.Table<any, any>}
     */
    readonly table: Dexie.Table<any, any>;
    /**
     * Get db ref
     *
     * @returns {Dexie}
     */
    readonly db: Dexie;
    init(coordinator: ICoordinator, opts: ICoordinatorOptions): Promise<ICoordinator>;
    start(): Promise<ICoordinator>;
    stop(): Promise<ICoordinator>;
    key(...args: any[]): IndexedDBKeyValue;
    keyFromObject(o: any): IndexedDBKeyValue;
    dbKeyFromKey(key: IndexedDBKeyValue): any;
    get(key: IndexedDBKeyValue): Promise<M>;
    save(model: M): Promise<M>;
    /**
     * Remove implementation
     *
     * @param key
     * @returns {Promise<void>}
     */
    remove(key: IndexedDBKeyValue): Promise<any>;
    count(): Promise<number>;
    /**
     * Bulk get
     *
     * @param keys
     * @returns {any}
     */
    bulkGet(...keys: IndexedDBKeyValue[]): Promise<M[]>;
    /**
     * Bulk save/put
     *
     * @param models
     * @returns {M[]}
     */
    bulkSave(...models: M[]): Promise<M[]>;
    /**
     * Bulk remove
     *
     * @param keys
     * @returns {IndexedDBKeyValue[]}
     */
    bulkRemove(...keys: IndexedDBKeyValue[]): Promise<any[]>;
}
